# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

from azure.cli.testsdk import ScenarioTest, record_only


@record_only()
class AzureReservationsTests(ScenarioTest):

    def _validate_reservation_order(self, reservation_order):
        self.assertIsNotNone(reservation_order)
        self.assertTrue(reservation_order['etag'])
        self.assertTrue(reservation_order['id'])
        self.assertTrue(reservation_order['name'])
        self.assertTrue(reservation_order['originalQuantity'])
        self.assertTrue(reservation_order['provisioningState'])
        self.assertTrue(reservation_order['requestDateTime'])
        self.assertTrue(reservation_order['reservations'])
        self.assertTrue(reservation_order['term'])
        self.assertTrue(reservation_order['type'])
        self.assertTrue(reservation_order['type'] == 'Microsoft.Capacity/reservationOrders')

    def _validate_reservation(self, reservation):
        self.assertIsNotNone(reservation)
        self.assertTrue(reservation['location'])
        self.assertTrue(len(reservation['location']) > 0)
        self.assertTrue(reservation['etag'])
        self.assertTrue(reservation['etag'] > 0)
        self.assertTrue(reservation['id'])
        self.assertTrue(reservation['name'])
        self.assertTrue(reservation['sku'])
        self.assertTrue(reservation['properties'])
        self.assertTrue(reservation['type'])
        self.assertTrue(reservation['type'] == 'Microsoft.Capacity/reservationOrders/reservations')

    def test_get_applied_reservation_order_ids(self):
        self.kwargs.update({
            'subscription': '00000000-0000-0000-0000-000000000000'
        })
        result = self.cmd('reservations reservation-order-id list --subscription-id {subscription}') \
                     .get_output_in_json()
        for order_id in result['reservationOrderIds']['value']:
            self.assertIn('/providers/Microsoft.Capacity/reservationorders/', order_id)

    def test_list_reservation_order(self):
        reservation_order_list = self.cmd('reservations reservation-order list').get_output_in_json()
        self.assertIsNotNone(reservation_order_list)
        for order in reservation_order_list:
            self._validate_reservation_order(order)
            self.assertIn('/providers/microsoft.capacity/reservationOrders/', order['id'])
            self.assertGreater(order['etag'], 0)
            for reservation in order['reservations']:
                self.assertTrue(reservation['id'])

    def test_get_reservation_order(self):
        self.kwargs.update({
            'reservation_order_id': "98e884a1-2e01-4c9a-987e-e4e8be8f2775"
        })
        command = 'reservations reservation-order show --reservation-order-id {reservation_order_id}'
        reservation_order = self.cmd(command).get_output_in_json()
        self._validate_reservation_order(reservation_order)
        self.assertIn('/providers/microsoft.capacity/reservationOrders/', reservation_order['id'])
        self.assertGreater(reservation_order['etag'], 0)

    def test_list_reservation(self):
        self.kwargs.update({
            'reservation_order_id': "98e884a1-2e01-4c9a-987e-e4e8be8f2775"
        })
        reservation_list = self.cmd('reservations reservation list --reservation-order-id {reservation_order_id}') \
                               .get_output_in_json()
        self.assertIsNotNone(reservation_list)
        for reservation in reservation_list:
            self.assertIn(self.kwargs['reservation_order_id'], reservation['name'])
            self.assertGreater(reservation['etag'], 0)
            self.assertEqual('Microsoft.Capacity/reservationOrders/reservations', reservation['type'])

    def test_get_reservation(self):
        self.kwargs.update({
            'reservation_order_id': "98e884a1-2e01-4c9a-987e-e4e8be8f2775",
            'reservation_id': 'de06a4f6-06a7-41c7-9bfb-822863669d05'
        })
        reservation = self.cmd('reservations reservation show  --reservation-order-id {reservation_order_id} '
                               '--reservation-id {reservation_id}').get_output_in_json()
        self.assertIn(self.kwargs['reservation_order_id'], reservation['name'])
        self.assertGreater(reservation['etag'], 0)
        self.assertGreater(reservation['properties']['quantity'], 0)
        self.assertEqual('Microsoft.Capacity/reservationOrders/reservations', reservation['type'])

    def test_list_reservation_history(self):
        self.kwargs.update({
            'reservation_order_id': "98e884a1-2e01-4c9a-987e-e4e8be8f2775",
            'reservation_id': 'de06a4f6-06a7-41c7-9bfb-822863669d05'
        })
        history = self.cmd('reservations reservation list-history --reservation-order-id {reservation_order_id}'
                           ' --reservation-id {reservation_id}').get_output_in_json()
        self.assertGreater(len(history), 0)
        for entry in history:
            self.assertGreater(entry['etag'], 0)
            name_format = '{}/{}'.format(self.kwargs['reservation_order_id'], self.kwargs['reservation_id'])
            self.assertIn(name_format, entry['name'])

    def test_get_catalog(self):
        self.kwargs.update({
            'subscription': '00000000-0000-0000-0000-000000000000',
            'type': 'SuseLinux'
        })
        catalog = self.cmd('reservations catalog show --subscription-id {subscription} --reserved-resource-type {type}').get_output_in_json()
        self.assertGreater(len(catalog), 0)
        for entry in catalog:
            self.assertGreater(len(entry['terms']), 0)
            self.assertGreater(len(entry['skuProperties']), 0)
            self.assertIsNotNone(entry['resourceType'])
            self.assertIsNotNone(entry['name'])

        self.kwargs.update({
            'subscription': '00000000-0000-0000-0000-000000000000',
            'type': 'SqlDatabases',
            'location': 'westus'
        })
        catalog = self.cmd('reservations catalog show --subscription-id {subscription} --reserved-resource-type {type} --location {location}').get_output_in_json()
        self.assertGreater(len(catalog), 0)
        for entry in catalog:
            self.assertGreater(len(entry['terms']), 0)
            self.assertGreater(len(entry['skuProperties']), 0)
            self.assertIsNotNone(entry['resourceType'])
            self.assertIsNotNone(entry['name'])

        self.kwargs.update({
            'subscription': '00000000-0000-0000-0000-000000000000',
            'type': 'VirtualMachines',
            'location': 'westus'
        })
        catalog = self.cmd('reservations catalog show --subscription-id {subscription} --reserved-resource-type {type} --location {location}').get_output_in_json()
        self.assertGreater(len(catalog), 0)
        for entry in catalog:
            self.assertGreater(len(entry['terms']), 0)
            self.assertGreater(len(entry['skuProperties']), 0)
            self.assertIsNotNone(entry['resourceType'])
            self.assertIsNotNone(entry['name'])

    def test_update_reservation(self):
        self.kwargs.update({
            'reservation_order_id': "98e884a1-2e01-4c9a-987e-e4e8be8f2775",
            'reservation_id': 'e208f907-6d08-4b07-b857-91071cac521b',
            'scope': '/subscriptions/302110e3-cd4e-4244-9874-07c91853c809',
            'instance_flexibility': "Off"
        })

        single_reservation = self.cmd('reservations reservation update --reservation-order-id {reservation_order_id}'
                                      ' --reservation-id {reservation_id} -t Single -s {scope}'
                                      ' --instance-flexibility {instance_flexibility}').get_output_in_json()
        self.assertEqual('Single', single_reservation['properties']['appliedScopeType'])

        shared_reservation = self.cmd('reservations reservation update --reservation-order-id {reservation_order_id} '
                                      '--reservation-id {reservation_id} -t Shared'
                                      ' --instance-flexibility {instance_flexibility}').get_output_in_json()
        self.assertEqual('Shared', shared_reservation['properties']['appliedScopeType'])

    def test_split_and_merge(self):
        self.kwargs.update({
            'reservation_order_id': "98e884a1-2e01-4c9a-987e-e4e8be8f2775",
            'reservation_id': '4ed87136-43a4-4b7f-9a54-52b31740eea3',
            'quantity1': 1,
            'quantity2': 1
        })

        original_reservation = self.cmd('reservations reservation show  --reservation-order-id {reservation_order_id}'
                                        ' --reservation-id {reservation_id}').get_output_in_json()
        original_quantity = original_reservation['properties']['quantity']

        split_items = self.cmd('reservations reservation split --reservation-order-id {reservation_order_id} '
                               '--reservation-id {reservation_id} -1 {quantity1} -2 {quantity2}').get_output_in_json()
        self.assertIsNotNone(split_items)

        quantity_sum = 0
        split_ids = []
        for item in split_items:
            self._validate_reservation(item)
            if 'Succeeded' in item['properties']['provisioningState']:
                item_id = item['name'].split('/')[1]
                split_ids.append(item_id)
                quantity_sum += item['properties']['quantity']
        self.assertEqual(original_quantity, quantity_sum)
        self.assertEqual(2, len(split_ids))

        self.kwargs.update({
            'split_id1': split_ids[0],
            'split_id2': split_ids[1]
        })
        merge_items = self.cmd('reservations reservation merge --reservation-order-id {reservation_order_id} -1 '
                               '{split_id1} -2 {split_id2}').get_output_in_json()
        self.assertIsNotNone(merge_items)
        for item in merge_items:
            self._validate_reservation(item)
            if 'Succeeded' in item['properties']['provisioningState']:
                self.assertEqual(quantity_sum, item['properties']['quantity'])



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

import unittest

from azure.cli.testsdk import (
    LiveScenarioTest, ScenarioTest, ResourceGroupPreparer)

from azure.cli.command_modules.advisor.custom import (
    _build_filter_string,
    _parse_operation_id,
    _parse_recommendation_uri)


class AzureAdvisorUnitTest(unittest.TestCase):

    def test_build_filter(self):
        ids = ['a', 'b', 'c']
        resource_group_name = 'r'
        category = 'cost'

        self.assertEqual(
            _build_filter_string(),
            None)
        self.assertEqual(
            _build_filter_string(ids=ids),
            "ResourceId eq 'a' or ResourceId eq 'b' or ResourceId eq 'c'")
        self.assertEqual(
            _build_filter_string(
                resource_group_name=resource_group_name),
            "ResourceGroup eq 'r'")
        self.assertEqual(
            _build_filter_string(
                category=category),
            "Category eq 'cost'")
        self.assertEqual(
            _build_filter_string(
                ids=ids,
                category=category),
            "(ResourceId eq 'a' or ResourceId eq 'b' or ResourceId eq 'c') and Category eq 'cost'")
        self.assertEqual(
            _build_filter_string(
                resource_group_name=resource_group_name,
                category=category),
            "(ResourceGroup eq 'r') and Category eq 'cost'")

    def test__parse_operation_id(self):
        location = ("https://management.azure.com/subscriptions/00000000-0000-0000-0000-000000000000/"
                    "providers/Microsoft.Advisor/generateRecommendations/a9544ca5-5837-4cb4-94d6-bad2b6e76320"
                    "?api-version=2017-04-19")
        operation_id = _parse_operation_id(location)
        self.assertEqual(operation_id, 'a9544ca5-5837-4cb4-94d6-bad2b6e76320')

    def test__parse_recommendation_uri(self):
        recommendation_uri = ("/subscriptions/00000000-0000-0000-0000-000000000000/resourceGroups/foo"
                              "/providers/Microsoft.Network/expressRouteCircuits/test/"
                              "providers/Microsoft.Advisor/recommendations/c4deb869-ea38-f90d-331f-91770021d425"
                              "/suppressions/5c9c3fce-c1b2-7e45-106c-152ce3c04be5")
        result = _parse_recommendation_uri(recommendation_uri)
        self.assertEqual(
            result['resource_uri'],
            ("/subscriptions/00000000-0000-0000-0000-000000000000/resourceGroups/foo"
             "/providers/Microsoft.Network/expressRouteCircuits/test"))
        self.assertEqual(result['recommendation_id'], 'c4deb869-ea38-f90d-331f-91770021d425')


class AzureAdvisorLiveScenarioTest(LiveScenarioTest):

    def test_recommendations(self):
        # List should return at least one recommendation with filters
        output = self.cmd('advisor recommendation list --category Security',
                          checks=self.check("[0].category", 'Security')).get_output_in_json()

        # Set the recommendation details to use with further commands
        rec = output[0]
        name = rec['name']
        recommendation_id = rec['id']
        resource_id = _parse_recommendation_uri(recommendation_id)
        self.kwargs.update({
            'name': name,
            'recommendation_id': recommendation_id,
            'resource_id': resource_id
        })

        # Disable with specified duration should create a suppression
        self.cmd('advisor recommendation disable --ids {recommendation_id} --days 1',
                 checks=[self.check("[0].id", recommendation_id),
                         self.exists("[0].suppressionIds")])

        # Disable again should create another suppression
        self.cmd('advisor recommendation disable --ids {recommendation_id}',
                 checks=[self.check("[0].id", recommendation_id),
                         self.greater_than("length([0].suppressionIds)", 1)])

        # Enable should remove all suppressions
        self.cmd('advisor recommendation enable --ids {recommendation_id}',
                 checks=[self.check("[0].id", recommendation_id),
                         self.check("[0].suppressionIds", None)])

        # Enable again should be a no op
        self.cmd('advisor recommendation enable --ids {recommendation_id}',
                 checks=[self.check("[0].id", recommendation_id),
                         self.check("[0].suppressionIds", None)])

    def test_recommendations_resource_group(self):
        resource_group = 'AzExpertStg'
        self.kwargs.update({
            'resource_group': resource_group
        })

        # List should return at least one recommendation with filters
        output = self.cmd('advisor recommendation list --resource-group {resource_group}',
                          checks=self.check("[0].resourceGroup", resource_group)).get_output_in_json()

        # Set the recommendation details to use with further commands
        resource_id = _parse_recommendation_uri(output[0]['id'])['resource_uri']
        name = output[0]['name']
        self.kwargs.update({
            'resource_id': resource_id,
            'name': name
        })

        # Disable with specified duration should create a suppression
        self.cmd('advisor recommendation disable -n {name} -g {resource_group} --days 1',
                 checks=[self.check("[0].name", name),
                         self.check("[0].resourceGroup", resource_group),
                         self.exists("[0].suppressionIds")])

        # Disable again should create another suppression
        self.cmd('advisor recommendation disable -n {name} -g {resource_group}',
                 checks=[self.check("[0].name", name),
                         self.check("[0].resourceGroup", resource_group),
                         self.greater_than("length([0].suppressionIds)", 1)])

        # Enable should remove all suppressions
        self.cmd('advisor recommendation enable -n {name} -g {resource_group}',
                 checks=[self.check("[0].name", name),
                         self.check("[0].resourceGroup", resource_group),
                         self.check("[0].suppressionIds", None)])

        # Enable again should be a no op
        self.cmd('advisor recommendation enable -n {name} -g {resource_group}',
                 checks=[self.check("[0].name", name),
                         self.check("[0].resourceGroup", resource_group),
                         self.check("[0].suppressionIds", None)])


class AzureAdvisorScenarioTest(ScenarioTest):

    def test_configurations_subscription(self):
        # Show should always return a default
        self.cmd('advisor configuration show',
                 checks=[self.check("type", "Microsoft.Advisor/Configurations")])

        # Show should reflect the changes made by Update
        self.cmd('advisor configuration update --low-cpu-threshold 15 --exclude')
        self.cmd('advisor configuration show',
                 checks=[self.check("properties.lowCpuThreshold", "15"),
                         self.check("properties.exclude", True)])

        # Show should reflect the changes made by Update
        self.cmd('advisor configuration update --low-cpu-threshold 5 --include')
        self.cmd('advisor configuration show',
                 checks=[self.check("properties.lowCpuThreshold", "5"),
                         self.check("properties.exclude", False)])

        # List should reflect the changes made by Update
        self.cmd('advisor configuration list',
                 checks=[self.check("[0].properties.lowCpuThreshold", "5"),
                         self.check("[0].properties.exclude", False)])

    @ResourceGroupPreparer(name_prefix='cli_test_advisor')
    def test_configurations_resource_group(self, resource_group):
        # Show should always return a default even for a brand new resource group
        self.cmd('advisor configuration show -g {rg}',
                 checks=[self.check("resourceGroup", resource_group)])

        # Show should reflect the changes made by Update
        self.cmd('advisor configuration update --exclude -g {rg}')
        self.cmd('advisor configuration show -g {rg}',
                 checks=[self.check("resourceGroup", resource_group),
                         self.check("properties.exclude", True)])

        # Show should reflect the changes made by Update
        self.cmd('advisor configuration update --include -g {rg}')
        self.cmd('advisor configuration show -g {rg}',
                 checks=[self.check("resourceGroup", resource_group),
                         self.check("properties.exclude", False)])


if __name__ == '__main__':
    unittest.main()



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

import contextlib
import unittest
import mock
import sys
import six
from six import StringIO

from azure.cli.command_modules.find.custom import _purge, find
from azure.cli.core.mock import DummyCli


@contextlib.contextmanager
def capture():
    oldout, olderr = sys.stdout, sys.stderr
    try:
        out = [StringIO(), StringIO()]
        sys.stdout, sys.stderr = out
        yield out
    finally:
        sys.stdout, sys.stderr = oldout, olderr
        out[0] = out[0].getvalue()
        out[1] = out[1].getvalue()


class SearchIndexTest(unittest.TestCase):
    def setUp(self):
        _purge()
        self.loader = mock.MagicMock()
        self.loader.cli_ctx = DummyCli()

    def execute(self, args, reindex=False):
        with capture() as out:
            find(self.loader, args, reindex)

        return out[0]

    def test_search_index(self):
        six.assertRegex(
            self,
            self.execute(['keyvault', 'list']),
            'az keyvault list')

    def test_search_reindex(self):
        six.assertRegex(
            self,
            self.execute(['keyvault', 'list'], reindex=True),
            'az keyvault list')



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------
from azure.cli.testsdk import ResourceGroupPreparer, JMESPathCheck
from azure.cli.testsdk import ScenarioTest
from azure.mgmt.cdn.models import SkuName
from .scenario_mixin import CdnScenarioMixin


class CdnProfileScenarioTest(CdnScenarioMixin, ScenarioTest):
    @ResourceGroupPreparer()
    def test_cdn_profile_crud(self, resource_group):
        list_checks = [JMESPathCheck('length(@)', 0)]
        self.profile_list_cmd(resource_group, checks=list_checks)

        profile_name = 'profile123'
        checks = [JMESPathCheck('name', profile_name),
                  JMESPathCheck('sku.name', SkuName.standard_akamai.value)]
        self.profile_create_cmd(resource_group, profile_name, checks=checks)

        list_checks = [JMESPathCheck('length(@)', 1)]
        self.profile_list_cmd(resource_group, checks=list_checks)

        checks.append(JMESPathCheck('tags.foo', 'bar'))
        self.profile_update_cmd(resource_group, profile_name, tags='foo=bar', checks=checks)

        self.profile_delete_cmd(resource_group, profile_name)


class CdnCustomDomainScenarioTest(ScenarioTest):

    @ResourceGroupPreparer(name_prefix='cli_test_cdn_domain')
    def test_cdn_custom_domain(self, resource_group):
        from msrestazure.azure_exceptions import CloudError
        from azure.mgmt.cdn.models import ErrorResponseException
        from knack.util import CLIError

        self.kwargs.update({
            'profile': 'cdnprofile1',
            'endpoint': self.create_random_name(prefix='endpoint', length=24),
            'origin': 'www.test.com',
            'hostname': 'www.example.com',
            'name': 'customdomain1'
        })

        self.cmd('cdn profile create -g {rg} -n {profile}')
        self.cmd('cdn endpoint create -g {rg} --origin {origin} --profile-name {profile} -n {endpoint}')
        self.cmd('cdn custom-domain list -g {rg} --endpoint-name {endpoint} --profile-name {profile}')

        # These will all fail because we don't really have the ability to create the custom endpoint in test.
        # but they should still fail if there was a CLI-level regression.
        with self.assertRaises(CloudError):
            self.cmd('cdn custom-domain create -g {rg} --endpoint-name {endpoint} --hostname {hostname} --profile-name {profile} -n {name}')
        with self.assertRaises(SystemExit):  # exits with code 3 due to missing resource
            self.cmd('cdn custom-domain show -g {rg} --endpoint-name {endpoint} --profile-name {profile} -n {name}')
        self.cmd('cdn custom-domain delete -g {rg} --endpoint-name {endpoint} --profile-name {profile} -n {name}')
        with self.assertRaises(ErrorResponseException):
            self.cmd('cdn custom-domain enable-https -g {rg} --endpoint-name {endpoint} --profile-name {profile} -n {name}')
        with self.assertRaises(ErrorResponseException):
            self.cmd('cdn custom-domain disable-https -g {rg} --endpoint-name {endpoint} --profile-name {profile} -n {name}')



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

import six
from azure.cli.command_modules.interactive.azclishell.util import parse_quotes
import unittest


class ParseTest(unittest.TestCase):

    def test_parse_quotes(self):
        cmd1 = 'account set --subscription \'Visual Studeio enterprizse (msft)\''
        args1 = parse_quotes(cmd1)
        self.assertEqual(
            args1, ['account', 'set', '--subscription', 'Visual Studeio enterprizse (msft)'])

        cmd2 = 'account set --subscription \"Visual Studeio enterprizse (msft)\"'
        args2 = parse_quotes(cmd2)
        self.assertEqual(
            args2, ['account', 'set', '--subscription', 'Visual Studeio enterprizse (msft)'])

        cmd3 = 'account set --subscription'
        args3 = parse_quotes(cmd3)
        self.assertEqual(args3, ['account', 'set', '--subscription'])

        cmd4 = 'word1 "quote1" word2 "quote is 3" words are fun'
        args4 = parse_quotes(cmd4)
        self.assertEqual(
            args4, ['word1', "quote1", 'word2', "quote is 3", 'words', 'are', 'fun'])

        cmd5 = '"??[?resourceGroup == "CJ101"].name"'
        args5 = parse_quotes(cmd5)
        self.assertEqual(
            args5, ['??[?resourceGroup == CJ101].name']
        )


if __name__ == '__main__':
    unittest.main()
